@@ -1,17 +1,3 @@
-setClassUnion("unmarkedFrameOrNULL", members=c("unmarkedFrame", "NULL"))
-
-setClass("unmarkedFitIDS",
-    representation(
-        formlist = "list",
-        keyfun = "character",
-        K = "numeric",
-        dataPC = "unmarkedFrameOrNULL",
-        dataOC = "unmarkedFrameOrNULL",
-        maxDist = "list",
-        surveyDurations = "list",
-        unitsOut = "character"),
-        contains = "unmarkedFit")
-
 get_ds_info <- function(db){
   J <- length(db) - 1
   a <- u <- rep(NA, J)
@@ -110,35 +96,45 @@ IDS <- function(lambdaformula = ~1,
   # Design matrices------------------------------------------------------------
 
   # Need to add offset support here eventually
-  gd_hds <- getDesign(dataDS, form_hds)
+  form_hds_split <- split_formula(form_hds)
+  names(form_hds_split) <- c("det", "state")
+  gd_hds <- getDesign(dataDS, form_hds_split)
   if(any(is.na(gd_hds$y))){
     stop("Missing values in only some distance bins is not supported", call.=FALSE)
   } 
   ds_hds <- get_ds_info(dataDS@dist.breaks)
   Xavail_ds <- matrix(0,0,0)
-  if(has_avail) Xavail_ds <- getDesign(dataDS, form_avail)$X
+  if(has_avail){
+    form_avail_split <- split_formula(form_avail)
+    names(form_avail_split) <- c("det", "state")
+    Xavail_ds <- getDesign(dataDS, form_avail_split)$X_state
+  }
   if(is.null(durationDS)) durationDS <- rep(0,0)
 
-  gd_pc <- list(y=matrix(0,0,0), X=matrix(0,0,0), V=matrix(0,0,0))
+  gd_pc <- list(y=matrix(0,0,0), X_state=matrix(0,0,0), X_det=matrix(0,0,0))
   ds_pc <- list(total_area=0, db=c(0,0), a=0, w=0, u=0)
   Xavail_pc <- matrix(0,0,0)
   if(is.null(durationPC)) durationPC <- rep(0,0)
   if(!is.null(dataPC)){
-    gd_pc <- getDesign(dataPC, form_pc)
+    form_pc_split <- split_formula(form_pc)
+    names(form_pc_split) <- c("det", "state")
+    gd_pc <- getDesign(dataPC, form_pc_split)
     ds_pc <- get_ds_info(c(0, maxDistPC))
-    if(has_avail) Xavail_pc <- getDesign(dataPC, form_avail)$X
+    if(has_avail) Xavail_pc <- getDesign(dataPC, form_avail_split)$X_state
   }
 
-  gd_oc <- list(y=matrix(0,0,0), X=matrix(0,0,0), V=matrix(0,0,0))
+  gd_oc <- list(y=matrix(0,0,0), X_state=matrix(0,0,0), X_det=matrix(0,0,0))
   ds_oc <- list(total_area=0, db=c(0,0), a=0, w=0, u=0)
   Kmin_oc <- rep(0,0)
   Xavail_oc <- matrix(0,0,0)
   if(is.null(durationOC)) durationOC <- rep(0,0)
   if(!is.null(dataOC)){
-    gd_oc <- getDesign(dataOC, form_oc)
+    form_oc_split <- split_formula(form_pc)
+    names(form_oc_split) <- c("det", "state")
+    gd_oc <- getDesign(dataOC, form_oc_split)
     ds_oc <- get_ds_info(c(0, maxDistOC))
     Kmin_oc <- apply(gd_oc$y, 1, max, na.rm=T)
-    if(has_avail) Xavail_oc <- getDesign(dataOC, form_avail)$X
+    if(has_avail) Xavail_oc <- getDesign(dataOC, form_avail_split)$X_state
   }
 
   # Density conversion and unequal area correction
@@ -157,13 +153,13 @@ IDS <- function(lambdaformula = ~1,
 
   # Parameter stuff------------------------------------------------------------
   pind_mat <- matrix(0, nrow=8, ncol=2)
-  pind_mat[1,] <- c(1, ncol(gd_hds$X))
-  pind_mat[2,] <- max(pind_mat) + c(1, ncol(gd_hds$V))
+  pind_mat[1,] <- c(1, ncol(gd_hds$X_state))
+  pind_mat[2,] <- max(pind_mat) + c(1, ncol(gd_hds$X_det))
   if(!is.null(detformulaPC) & !is.null(dataPC)){
-    pind_mat[3,] <- max(pind_mat) + c(1, ncol(gd_pc$V))
+    pind_mat[3,] <- max(pind_mat) + c(1, ncol(gd_pc$X_det))
   }
   if(!is.null(detformulaOC) & !is.null(dataOC)){
-    pind_mat[4,] <- max(pind_mat) + c(1, ncol(gd_oc$V))
+    pind_mat[4,] <- max(pind_mat) + c(1, ncol(gd_oc$X_det))
   }
   if(has_avail){
     pind_mat[5,] <- max(pind_mat) + c(1, ncol(Xavail_ds))
@@ -181,8 +177,8 @@ IDS <- function(lambdaformula = ~1,
 
   if(is.null(starts)){
     lam_init <- log(mean(apply(dataDS@y, 1, sum, na.rm=TRUE)) / lam_adjust[1])
-    params_tmb <- list(beta_lam = c(lam_init, rep(0, ncol(gd_hds$X)-1)),
-                     beta_hds = c(log(median(dataDS@dist.breaks)),rep(0, ncol(gd_hds$V)-1)),
+    params_tmb <- list(beta_lam = c(lam_init, rep(0, ncol(gd_hds$X_state)-1)),
+                     beta_hds = c(log(median(dataDS@dist.breaks)),rep(0, ncol(gd_hds$X_det)-1)),
                      beta_pc = rep(0,0),
                      beta_oc = rep(0,0),
                      beta_avail = rep(0,0),
@@ -193,11 +189,11 @@ IDS <- function(lambdaformula = ~1,
     if(keyfun == "hazard") params_tmb$beta_schds <- 0
     
     if(!is.null(detformulaPC) & !is.null(dataPC)){
-      params_tmb$beta_pc <- c(log(maxDistPC/2), rep(0, ncol(gd_pc$V)-1))
+      params_tmb$beta_pc <- c(log(maxDistPC/2), rep(0, ncol(gd_pc$X_det)-1))
       if(keyfun == "hazard") params_tmb$beta_scpc <- 0
     }
     if(!is.null(detformulaOC) & !is.null(dataOC)){
-      params_tmb$beta_oc <- c(log(maxDistOC/2), rep(0, ncol(gd_oc$V)-1))
+      params_tmb$beta_oc <- c(log(maxDistOC/2), rep(0, ncol(gd_oc$X_det)-1))
       if(keyfun == "hazard") params_tmb$beta_scoc <- 0
     }
     if(has_avail){
@@ -238,16 +234,16 @@ IDS <- function(lambdaformula = ~1,
     pind = pind_mat, lam_adjust = lam_adjust,
 
     # HDS data
-    y_hds = gd_hds$y, X_hds = gd_hds$X, V_hds = gd_hds$V, key_hds = keyidx,
+    y_hds = gd_hds$y, X_hds = gd_hds$X_state, V_hds = gd_hds$X_det, key_hds = keyidx,
     db_hds = dataDS@dist.breaks, a_hds = ds_hds$a, w_hds = ds_hds$w,
     u_hds = ds_hds$u,
 
     # PC data
-    y_pc = gd_pc$y, X_pc = gd_pc$X, V_pc = gd_pc$V, key_pc = keyidx,
+    y_pc = gd_pc$y, X_pc = gd_pc$X_state, V_pc = gd_pc$X_det, key_pc = keyidx,
     db_pc = c(0, maxDistPC), a_pc = ds_pc$a, w_pc = ds_pc$w, u_pc = ds_pc$u,
 
     # occ data
-    y_oc = gd_oc$y, X_oc = gd_oc$X, V_oc = gd_oc$V, key_oc = keyidx,
+    y_oc = gd_oc$y, X_oc = gd_oc$X_state, V_oc = gd_oc$X_det, key_oc = keyidx,
     db_oc = c(0, maxDistOC), a_oc = ds_oc$a, w_oc = ds_oc$w, u_oc = ds_oc$u,
     K_oc = K, Kmin_oc = Kmin_oc,
 
@@ -265,37 +261,37 @@ IDS <- function(lambdaformula = ~1,
 
   sdr <- TMB::sdreport(tmb_obj)
 
-  lam_coef <- get_coef_info(sdr, "lam", colnames(gd_hds$X),
+  lam_coef <- get_coef_info(sdr, "lam", colnames(gd_hds$X_state),
                                        pind_mat[1,1]:pind_mat[1,2])
 
   lam_est <- unmarkedEstimate(name="Density", short.name="lam",
-    estimates = lam_coef$ests, covMat = lam_coef$cov, fixed=1:ncol(gd_hds$X),
+    estimates = lam_coef$ests, covMat = lam_coef$cov, fixed=1:ncol(gd_hds$X_state),
     invlink = "exp", invlinkGrad = "exp")
 
-  dist_coef <- get_coef_info(sdr, "hds", colnames(gd_hds$V),
+  dist_coef <- get_coef_info(sdr, "hds", colnames(gd_hds$X_det),
                                        pind_mat[2,1]:pind_mat[2,2])
 
   dist_est <- unmarkedEstimate(name="Distance sampling detection", short.name="ds",
-    estimates = dist_coef$ests, covMat = dist_coef$cov, fixed=1:ncol(gd_hds$V),
+    estimates = dist_coef$ests, covMat = dist_coef$cov, fixed=1:ncol(gd_hds$X_det),
     invlink = "exp", invlinkGrad = "exp")
 
   est_list <- list(lam=lam_est, ds=dist_est)
 
   if(!is.null(detformulaPC) & !is.null(dataPC)){
-    pc_coef <- get_coef_info(sdr, "pc", colnames(gd_pc$V),
+    pc_coef <- get_coef_info(sdr, "pc", colnames(gd_pc$X_det),
                                         pind_mat[3,1]:pind_mat[3,2])
 
     pc_est <- unmarkedEstimate(name="Point count detection", short.name="pc",
-      estimates = pc_coef$ests, covMat = pc_coef$cov, fixed=1:ncol(gd_pc$V),
+      estimates = pc_coef$ests, covMat = pc_coef$cov, fixed=1:ncol(gd_pc$X_det),
       invlink = "exp", invlinkGrad = "exp")
     est_list <- c(est_list, list(pc=pc_est))
   }
 
   if(!is.null(detformulaOC) & !is.null(dataOC)){
-    oc_coef <- get_coef_info(sdr, "oc", colnames(gd_oc$V),
+    oc_coef <- get_coef_info(sdr, "oc", colnames(gd_oc$X_det),
                                         pind_mat[4,1]:pind_mat[4,2])
     oc_est <- unmarkedEstimate(name="Presence/absence detection", short.name="oc",
-      estimates = oc_coef$ests, covMat = oc_coef$cov, fixed=1:ncol(gd_oc$V),
+      estimates = oc_coef$ests, covMat = oc_coef$cov, fixed=1:ncol(gd_oc$X_det),
       invlink = "exp", invlinkGrad = "exp")
     est_list <- c(est_list, list(oc=oc_est))
   }
@@ -336,7 +332,7 @@ IDS <- function(lambdaformula = ~1,
   est_list <- unmarkedEstimateList(est_list)
 
   new("unmarkedFitIDS", fitType = "IDS", call = match.call(),
-    opt = opt, formula = lambdaformula, formlist=formlist,
+    opt = opt, formlist=formlist,
     data = dataDS, dataPC=dataPC, dataOC=dataOC, K=K,
     surveyDurations=surveyDurations,
     maxDist = list(pc=maxDistPC, oc=maxDistOC),
@@ -414,43 +410,15 @@ IDS_convert_class <- function(inp, type, ds_type=NULL){
 
   form <- inp@formlist[[type]]
   if(type=="phi") form <- as.formula(paste(c(as.character(form), "~1"), collapse=""))
+  formlist <- split_formula(form)
+  names(formlist) <- c("det", "state")
 
-  new("unmarkedFitDS", fitType="IDS", opt=inp@opt, formula=form,
+  new("unmarkedFitDS", fitType="IDS", opt=inp@opt, formula=form, formlist=formlist,
       data=data, keyfun=inp@keyfun, unitsOut=inp@unitsOut,
       estimates=unmarkedEstimateList(est),
       AIC=inp@AIC, output="density", TMB=inp@TMB)
 }
 
-# This predict_internal method uses IDS_convert_class to allow pass-through to
-# distsamp predict method
-setMethod("predict_internal", "unmarkedFitIDS", function(object, type, newdata,
-          backTransform=TRUE, na.rm=FALSE, appendData=FALSE, level=0.95, re.form=NULL, ...){
-  stopifnot(type %in% names(object))
-
-  # Special case of phi and  no newdata
-  # We need a separate prediction for each detection dataset
-  if(type == "phi" & missing(newdata)){
-
-    dists <- names(object)[names(object) %in% c("ds", "pc", "oc")]
-    out <- lapply(dists, function(x){
-      conv <- IDS_convert_class(object, "phi", ds_type=x)
-      predict(conv, "det", backTransform=backTransform, appendData=appendData,
-              level=level, ...)
-    })
-    names(out) <- dists
-
-  } else { # Regular situation
-    conv <- IDS_convert_class(object, type)
-    type <- switch(type, lam="state", ds="det", pc="det", oc="det", phi="det")
-    out <- predict(conv, type=type, newdata=newdata, backTransform=backTransform, appendData=appendData,
-                   level=level, ...)
-  }
-  out
-})
-
-# Get availability probability
-setGeneric("getAvail", function(object, ...) standardGeneric("getAvail"))
-
 # Get availability for each data type and site as a probability
 setMethod("getAvail", "unmarkedFitIDS", function(object, ...){
   stopifnot("phi" %in% names(object))
@@ -463,46 +431,6 @@ setMethod("getAvail", "unmarkedFitIDS", function(object, ...){
   out
 })
 
-# Fitted method returns a list of matrices, one per data type
-setMethod("fitted_internal", "unmarkedFitIDS", function(object){
-
-  dists <- names(object)[names(object) %in% c("ds", "pc")]
-
-  # If there is an availability model, get availability
-  # Otherwise set it to 1
-  avail <- list(ds=1, pc=1, oc=1)
-  if("phi" %in% names(object)){
-    avail <- getAvail(object)
-  }
-
-  # fitted for distance and N-mix data components
-  out <- lapply(dists, function(x){
-    conv <- IDS_convert_class(object, type=x)
-    fitted(conv) * avail[[x]]
-  })
-  names(out) <- dists
-
-  # fitted for occupancy data
-  if("oc" %in% names(object)){
-    conv <- IDS_convert_class(object, type="oc")
-    lam <- predict(conv, 'state', level = NULL, na.rm=FALSE)$Predicted
-    A <- pi*max(conv@data@dist.breaks)^2
-    switch(conv@data@unitsIn,
-            m = A <- A / 1e6,
-            km = A <- A)
-    switch(conv@unitsOut,
-            m = A <- A * 1e6,
-            ha = A <- A * 100,
-            kmsq = A <- A)
-    lam <- lam * A
-
-    p <- getP(conv, na.rm=FALSE) * avail$oc
-    out$oc <- 1 - exp(-lam*p) ## analytical integration.
-  }
-
-  out
-})
-
 # getP returns detection probability WITHOUT availability
 setMethod("getP_internal", "unmarkedFitIDS", function(object){
 
@@ -516,28 +444,6 @@ setMethod("getP_internal", "unmarkedFitIDS", function(object){
   out
 })
 
-
-setMethod("residuals_internal", "unmarkedFitIDS", function(object){
-
-  dists <- names(object)[names(object) %in% c("ds", "pc")]
-
-  # distance and N-mix data
-  out <- lapply(dists, function(x){
-    conv <- IDS_convert_class(object, type=x)
-    residuals(conv)
-  })
-  names(out) <- dists
-
-  # occupancy data
-  if("oc" %in% names(object)){
-    y <- object@dataOC@y
-    ft <- fitted(object)$oc
-    out$oc <- y - ft
-  }
-
-  out
-})
-
 setMethod("hist", "unmarkedFitIDS", function(x, lwd=1, lty=1, ...){
 
   conv <- IDS_convert_class(x, type='ds')
