@@ -10,7 +10,9 @@ occu <- function(formula, data, knownOcc = numeric(0),
     stop("Data is not an unmarkedFrameOccu object.")
 
   engine <- match.arg(engine, c("C", "R", "TMB"))
-  if(any(sapply(split_formula(formula), has_random))) engine <- "TMB"
+  formulas <- split_formula(formula)
+  names(formulas) <- c("det", "state")
+  if(any(sapply(formulas, has_random))) engine <- "TMB"
   if(length(knownOcc)>0 & engine == "TMB"){
     stop("TMB engine does not support knownOcc argument", call.=FALSE)
   }
@@ -21,12 +23,8 @@ occu <- function(formula, data, knownOcc = numeric(0),
   psiLinkGrad <- ifelse(linkPsi=="cloglog", "cloglog.grad", "logistic.grad")
 
   # Format input data----------------------------------------------------------
-  designMats <- getDesign(data, formula)
-  y <- truncateToBinary(designMats$y)
-  X <- designMats$X; V <- designMats$V;
-  Z_state <- designMats$Z_state; Z_det <- designMats$Z_det
-  removed <- designMats$removed.sites
-  X.offset <- designMats$X.offset; V.offset <- designMats$V.offset
+  dm <- getDesign(data, formulas)
+  y <- truncateToBinary(dm$y)
 
   # Re-format some variables for C and R engines
   yvec <- as.numeric(t(y))
@@ -36,13 +34,13 @@ occu <- function(formula, data, knownOcc = numeric(0),
   # convert knownOcc to logical so we can correctly to handle NAs.
   knownOccLog <- rep(FALSE, numSites(data))
   knownOccLog[knownOcc] <- TRUE
-  if(length(removed)>0) knownOccLog <- knownOccLog[-removed]
+  if(length(dm$removed.sites)>0) knownOccLog <- knownOccLog[-dm$removed.sites]
 
   # Set up parameter names and indices-----------------------------------------
-  occParms <- colnames(X)
-  detParms <- colnames(V)
-  nDP <- ncol(V)
-  nOP <- ncol(X)
+  occParms <- colnames(dm$X_state)
+  detParms <- colnames(dm$X_det)
+  nDP <- ncol(dm$X_det)
+  nOP <- ncol(dm$X_state)
   nP <- nDP + nOP
   psiIdx <- 1:nOP
   pIdx <- (nOP+1):nP
@@ -53,8 +51,8 @@ occu <- function(formula, data, knownOcc = numeric(0),
       beta.psi <- params[1:nOP]
       beta.p <- params[(nOP+1):nP]
       nll_occu(
-        yvec, X, V, beta.psi, beta.p, nd, knownOccLog, navec,
-        X.offset, V.offset, linkPsi
+        yvec, dm$X_state, dm$X_det, beta.psi, beta.p, nd, knownOccLog, navec,
+        dm$offset_state, dm$offset_det, linkPsi
       )
     }
   } else if (identical(engine, "R")){
@@ -63,9 +61,9 @@ occu <- function(formula, data, knownOcc = numeric(0),
     M <- nrow(y)
 
     nll <- function(params) {
-      psi <- psiLinkFunc(X %*% params[1 : nOP] + X.offset)
+      psi <- psiLinkFunc(dm$X_state %*% params[1 : nOP] + dm$offset_state)
       psi[knownOccLog] <- 1
-      pvec <- plogis(V %*% params[(nOP + 1) : nP] + V.offset)
+      pvec <- plogis(dm$X_det %*% params[(nOP + 1) : nP] + dm$offset_det)
       cp <- (pvec^yvec) * ((1 - pvec)^(1 - yvec))
       cp[navec] <- 1 # so that NA's don't modify likelihood
       cpmat <- matrix(cp, M, J, byrow = TRUE) #
@@ -99,13 +97,12 @@ occu <- function(formula, data, knownOcc = numeric(0),
   } else if(identical(engine, "TMB")){
 
     # Set up TMB input data
-    forms <- split_formula(formula)
     obs_all <- add_covariates(obsCovs(data), siteCovs(data), length(getY(data)))
-    inps <- get_ranef_inputs(forms, list(det=obs_all, state=siteCovs(data)),
-                             list(V, X), designMats[c("Z_det","Z_state")])
+    inps <- get_ranef_inputs(formulas, list(det=obs_all, state=siteCovs(data)),
+                             list(dm$X_det, dm$X_state), dm[c("Z_det","Z_state")])
 
     tmb_dat <- c(list(y=y, no_detect=nd, link=ifelse(linkPsi=="cloglog",1,0),
-                      offset_state=X.offset, offset_det=V.offset), inps$data)
+                      offset_state=dm$offset_state, offset_det=dm$offset_det), inps$data)
 
     # Fit model with TMB
     if(missing(starts)) starts <- NULL
@@ -121,9 +118,9 @@ occu <- function(formula, data, knownOcc = numeric(0),
     det_coef <- get_coef_info(tmb_out$sdr, "det", detParms, (nOP+1):nP)
 
     # Organize random effect estimates
-    state_rand_info <- get_randvar_info(tmb_out$sdr, "state", forms[[2]],
+    state_rand_info <- get_randvar_info(tmb_out$sdr, "state", formulas$state,
                                         siteCovs(data))
-    det_rand_info <- get_randvar_info(tmb_out$sdr, "det", forms[[1]],
+    det_rand_info <- get_randvar_info(tmb_out$sdr, "det", formulas$det,
                                       obs_all)
 
     }
@@ -151,8 +148,8 @@ occu <- function(formula, data, knownOcc = numeric(0),
 
   # Create unmarkedFit object--------------------------------------------------
   umfit <- new("unmarkedFitOccu", fitType = "occu", call = match.call(),
-                 formula = formula, data = data,
-                 sitesRemoved = designMats$removed.sites,
+                 formula = formula, formlist = formulas, data = data,
+                 sitesRemoved = dm$removed.sites,
                  estimates = estimateList, AIC = fmAIC, opt = fm,
                  negLogLike = fm$value,
                  nllFun = nll, knownOcc = knownOccLog, TMB=tmb_mod)
