@@ -11,7 +11,9 @@ pcount.spHDS<-
 function (formula, data, K, mixture = c("P", "NB", "ZIP"), starts,
     method = "BFGS", se = TRUE, ...)
 {
-    check_no_support(split_formula(formula))
+    formulas <- split_formula(formula)
+    names(formulas) <- c("det", "state")
+    check_no_support(formulas)
 
     mixture <- match.arg(mixture, c("P", "NB", "ZIP"))
     if (!is(data, "unmarkedFramePCount"))
@@ -20,25 +22,16 @@ function (formula, data, K, mixture = c("P", "NB", "ZIP"), starts,
    #engine <- match.arg(engine, c("C", "R"))
     if (identical(mixture, "ZIP") & identical(engine, "R"))
         stop("ZIP mixture not available for engine='R'")
-    designMats <- getDesign(data, formula)
+    dm <- getDesign(data, formulas)
+    y <- dm$y
 
-    X <- designMats$X
-    V <- designMats$V
-    y <- designMats$y
-    X.offset <- designMats$X.offset
-    V.offset <- designMats$V.offset
-
-    if (is.null(X.offset))
-        X.offset <- rep(0, nrow(X))
-    if (is.null(V.offset))
-        V.offset <- rep(0, nrow(V))
     NAmat <- is.na(y)
     J <- ncol(y)
     M <- nrow(y)
-    lamParms <- colnames(X)
-    detParms <- colnames(V)
-    nDP <- ncol(V)
-    nAP <- ncol(X)
+    lamParms <- colnames(dm$X_state)
+    detParms <- colnames(dm$X_det)
+    nDP <- ncol(dm$X_det)
+    nAP <- ncol(dm$X_state)
     if (missing(K)) {
         K <- max(y, na.rm = TRUE) + 100
         warning("K was not specified and was set to ", K, ".")
@@ -68,9 +61,9 @@ function (formula, data, K, mixture = c("P", "NB", "ZIP"), starts,
             tmp.parms[1]<- -1*exp(tmp.parms[1])
             parms[(nAP + 1):(nAP + nDP)]<- tmp.parms
 
-            theta.i <- exp(X %*% parms[1:nAP] + X.offset)
-            p.ij <- 2*plogis(V %*% (parms[(nAP + 1):(nAP + nDP)]) +
-                V.offset)
+            theta.i <- exp(dm$X_state %*% parms[1:nAP] + dm$offset_state)
+            p.ij <- 2*plogis(dm$X_det %*% (parms[(nAP + 1):(nAP + nDP)]) +
+                dm$offset_det)
             theta.ik <- rep(theta.i, each = K + 1)
             p.ijk <- rep(p.ij, each = K + 1)
             bin.ijk <- dbinom(y.ijk, k.ijk, p.ijk)
@@ -120,7 +113,7 @@ function (formula, data, K, mixture = c("P", "NB", "ZIP"), starts,
                 nP]), invlink = "logistic", invlinkGrad = "logistic.grad")
     }
     umfit <- new("unmarkedFitPCount", fitType = "pcount", call = match.call(),
-        formula = formula, data = data, sitesRemoved = designMats$removed.sites,
+        formula = formula, formlist = formulas, data = data, sitesRemoved = dm$removed.sites,
         estimates = estimateList, AIC = fmAIC, opt = fm, negLogLike = fm$value,
         nllFun = nll, K = K, mixture = mixture)
     return(umfit)
