@@ -24,7 +24,8 @@ test_that("unmarkedFramePCount subset works", {
   expect_equivalent(umf_sub[3,], umf[4,])
 
   keep <- c(FALSE, FALSE, TRUE, FALSE, TRUE)
-  expect_error(umf_sub <- umf[keep,]) # this should work
+  umf_sub <- umf[keep,]
+  expect_equal(umf_sub, umf[c(3,5),])
 })
 
 test_that("pcount can fit simple models",{
@@ -150,11 +151,11 @@ test_that("pcount predict works",{
 
   fm2 <- pcount(~1 ~1, umf1, K=40, mixture="NB")
   E2.1 <- predict(fm2, type="state")
-  expect_error(predict(fm2, type="alpha"))
+  expect_equal(predict(fm2, type="alpha")$Predicted, exp(unname(coef(fm2))[3]))
 
   fm3 <- pcount(~1 ~1, umf1, K=40, mixture="ZIP")
   E3.1 <- predict(fm3, type="state")
-  expect_error(predict(fm3, type="psi"))
+  expect_equal(predict(fm3, type="psi")$Predicted, plogis(unname(coef(fm3))[3]))
   expect_equal(E3.1[1,1], 1.818512, tol=1e-6)
 
 })
@@ -278,3 +279,34 @@ test_that("pcount R, C++ and TMB engines give same results",{
   expect_equal(coef(fmC), coef(fmR))
   expect_equal(coef(fmC), coef(fmT), tol=1e-7)
 })
+
+test_that("parboot with ZIP distribution works", {
+  #https://github.com/ecoverseR/unmarked/issues/49
+  set.seed(123)
+  nSites <- 100
+  nVisits <- 4
+  psi <- 0.5   # Proportion of extra-Poisson zeros
+  z <- rbinom(n=nSites, size=1, prob=psi) # Extra zeros
+  lam <- 5     # expected count when z=1
+  N <- rpois(n=nSites, lambda=lam*z)      # abundance at each site
+  p <- 0.3     # detection prob
+  y <- matrix(NA, nSites, nVisits)
+  for(i in 1:nSites) {
+    y[i,] <- rbinom(n=nVisits, size=N[i], prob=p) # count data
+  }
+
+  umf <- unmarkedFramePCount(y=y)
+  fmzip <- pcount(~1~1, umf, mixture="ZIP", K=25)
+
+  # Check simulate/parboot
+  pbzip <- parboot(fmzip)
+  expect_equal(pbzip@t.star[1,1], 668.8506, tol=1e-4)
+  expect_true(pbzip@t0 > min(pbzip@t.star) & pbzip@t0 < max(pbzip@t.star)) 
+  
+  # Check ranef
+  r <- ranef(fmzip)
+  b <- bup(r)
+  # plot(b, N)
+  # abline(a=0, b=1)
+  expect_equal(mean(b), 2.199303, tol=1e-5) 
+})
