@@ -3,13 +3,13 @@ context("predict-related functions")
 skip_on_cran()
 
 set.seed(123)
-cf <- list(state=c(intercept=0, elev=0.4, groupB=-0.5, groupC=0.6),
-           det=c(intercept=0))
-des <- list(M=100, J=5)
-guide <- list(group=factor(levels=c("A","B","C")))
-forms <- list(state=~elev+group, det=~1)
-
-umf <- expect_warning(simulate("occu", design=des, formulas=forms, coefs=cf, guide=guide))
+umf <- unmarkedFrameOccu(
+  y = matrix(NA, 100, 5),
+  siteCovs = data.frame(elev = rnorm(100),
+                        group = factor(sample(c("A","B","C"), 100, replace=TRUE)))
+)
+umf <- simulate(umf, model = occu, formula=~1~elev+group,
+                coefs = list(state = c(0, 0.4, -0.5, 0.6), det = 0))[[1]]
 mod <- occu(~1~elev+group, umf)
 
 test_that("clean_up_covs works with dynamic model data",{
@@ -34,7 +34,8 @@ test_that("clean_up_covs works with dynamic model data",{
                                                  sc1=c(1,NA,2,NA,3,NA)))
   expect_equivalent(dr$obs_covs, data.frame(oc1=1:12, ysc1=rep(1:6, each=2),
                                        ysc2=factor(rep(c("a","b"), each=2)),
-                                       sc1=rep(1:3, each=4)))
+                                       sc1=rep(1:3, each=4),
+                                       obsNum=factor(rep(1:4,3))))
 
   no_drop <- unmarked:::clean_up_covs(umf)
   expect_equivalent(no_drop$yearly_site_covs, data.frame(ysc1=1:6,
@@ -45,7 +46,8 @@ test_that("clean_up_covs works with dynamic model data",{
 
   cc <- unmarked:::clean_up_covs(umf, drop_final=TRUE)
   expect_equivalent(cc$obs_covs,
-                    data.frame(.dummy3=rep(1,12), .dummy2=rep(1,12), .dummy1=rep(1,12)))
+                    data.frame(.dummy3=rep(1,12), .dummy2=rep(1,12), .dummy1=rep(1,12),
+                               obsNum=factor(rep(1:4, 3))))
 })
 
 test_that("clean_up_covs works with single-season models",{
@@ -55,7 +57,8 @@ test_that("clean_up_covs works with single-season models",{
   cc <- unmarked:::clean_up_covs(umf)
   expect_equal(names(cc), c("site_covs","obs_covs"))
   expect_equivalent(cc$site_covs, data.frame(sc1=1:3))
-  expect_equivalent(cc$obs_covs, data.frame(oc1=1:6, sc1=rep(1:3, each=2)))
+  expect_equivalent(cc$obs_covs, data.frame(oc1=1:6, sc1=rep(1:3, each=2),
+                                            obsNum = factor(rep(1:2, 3))))
   cc2 <- unmarked:::clean_up_covs(umf, drop_final=TRUE)
   expect_equal(cc, cc2)
 })
@@ -72,7 +75,7 @@ test_that("clean_up_covs works with models with no obs covs",{
 
   cc <- unmarked:::clean_up_covs(ltUMF)
   expect_equal(names(cc), c("site_covs", "obs_covs"))
-  expect_equal(dim(cc$obs_covs), c(12,4))
+  expect_equal(dim(cc$obs_covs), c(12,5))
 })
 
 test_that("clean_up_covs works with models where length(y) != length(p)",{
@@ -97,7 +100,8 @@ test_that("clean_up_covs works with models where length(y) != length(p)",{
   cc <- unmarked:::clean_up_covs(umf)
   expect_equivalent(cc$site_covs, data.frame(sc=1:3))
   expect_equivalent(cc$obs_covs, data.frame(observer=factor(c(rep(c("A","B"), 3))),
-                                            sc1=rep(1:3, each=2)))
+                                            sc1=rep(1:3, each=2),
+                                            obsNum=factor(rep(1:2, 3))))
 
 })
 
@@ -208,3 +212,18 @@ test_that("predicting from terra::rast works",{
   expect_is(predict(mod2, 'state', newdata=nd_raster), 'SpatRaster')
   expect_is(predict(mod2, 'state', newdata=nd_raster, re.form=NA), 'SpatRaster')
 })
+
+test_that("Warning when predicting and there are nested functions in formula", { 
+  nd <- data.frame(elev=c(0,1), group="B")
+  mod2 <- occu(~1~scale(elev)+group, umf)
+  expect_no_warning(predict(mod2, type='state', newdata=nd))
+
+  mod3 <- occu(~1~I(elev^2)+group, umf)
+  expect_no_warning(predict(mod3, type='state', newdata=nd))
+
+  mod4 <- occu(~1~I(scale(elev)^2)+group, umf)
+  expect_warning(predict(mod4, type='state', newdata=nd), "probably incorrect")
+
+  mod5 <- occu(~1~scale(I(elev^2))+group, umf)
+  expect_warning(predict(mod5, type='state', newdata=nd), "probably incorrect")
+})
